import {
  computed,
  defineComponent,
  h,
  onBeforeUnmount,
  onMounted,
  ref,
  resolveComponent
} from "vue";
import { useRouter as useVueRouter } from "vue-router";
import {
  hasProtocol,
  parseQuery,
  parseURL,
  withTrailingSlash,
  withoutTrailingSlash
} from "ufo";
import {
  navigateTo,
  onNuxtReady,
  preloadRouteComponents,
  useNuxtApp,
  useRouter
} from "nuxt/app";
const firstNonUndefined = (...args) => args.find((arg) => arg !== void 0);
const DEFAULT_EXTERNAL_REL_ATTRIBUTE = "noopener noreferrer";
export const requestIdleCallback = import.meta.server ? (
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  () => {
  }
) : globalThis.requestIdleCallback || ((cb) => {
  const start = Date.now();
  const idleDeadline = {
    didTimeout: false,
    timeRemaining: () => Math.max(0, 50 - (Date.now() - start))
  };
  return setTimeout(() => {
    cb(idleDeadline);
  }, 1);
});
export const cancelIdleCallback = import.meta.server ? (
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  () => {
  }
) : globalThis.cancelIdleCallback || ((id) => {
  clearTimeout(id);
});
/*! @__NO_SIDE_EFFECTS__ */
// @__NO_SIDE_EFFECTS__
export function defineNuxtLink(options) {
  const componentName = options.componentName || "NuxtLink";
  const checkPropConflicts = (props, main, sub) => {
    if (import.meta.dev && props[main] !== void 0 && props[sub] !== void 0)
      console.warn(
        `[${componentName}] \`${main}\` and \`${sub}\` cannot be used together. \`${sub}\` will be ignored.`
      );
  };
  const resolveTrailingSlashBehavior = (to, resolve) => {
    if (!to || options.trailingSlash !== "append" && options.trailingSlash !== "remove")
      return to;
    const normalizeTrailingSlash = options.trailingSlash === "append" ? withTrailingSlash : withoutTrailingSlash;
    if (typeof to === "string") return normalizeTrailingSlash(to, true);
    const path = "path" in to ? to.path : resolve(to).path;
    return {
      ...to,
      name: void 0,
      // named routes would otherwise always override trailing slash behavior
      path: normalizeTrailingSlash(path, true)
    };
  };
  return defineComponent({
    name: componentName,
    props: {
      // Routing
      to: {
        type: [String, Object],
        default: void 0,
        required: false
      },
      href: {
        type: [String, Object],
        default: void 0,
        required: false
      },
      // Attributes
      target: {
        type: String,
        default: void 0,
        required: false
      },
      rel: {
        type: String,
        default: void 0,
        required: false
      },
      noRel: {
        type: Boolean,
        default: void 0,
        required: false
      },
      // Prefetching
      prefetch: {
        type: Boolean,
        default: void 0,
        required: false
      },
      noPrefetch: {
        type: Boolean,
        default: void 0,
        required: false
      },
      // Styling
      activeClass: {
        type: String,
        default: void 0,
        required: false
      },
      exactActiveClass: {
        type: String,
        default: void 0,
        required: false
      },
      prefetchedClass: {
        type: String,
        default: void 0,
        required: false
      },
      // Vue Router's `<RouterLink>` additional props
      replace: {
        type: Boolean,
        default: void 0,
        required: false
      },
      ariaCurrentValue: {
        type: String,
        default: void 0,
        required: false
      },
      // Edge cases handling
      external: {
        type: Boolean,
        default: void 0,
        required: false
      },
      // Slot API
      custom: {
        type: Boolean,
        default: void 0,
        required: false
      }
    },
    setup(props, { slots }) {
      const router = useRouter() ?? useVueRouter();
      const to = computed(() => {
        checkPropConflicts(props, "to", "href");
        const path = props.to || props.href || "";
        return resolveTrailingSlashBehavior(path, router.resolve);
      });
      const isExternal = computed(() => {
        if (props.external) return true;
        if (props.target && props.target !== "_self") return true;
        if (typeof to.value === "object") return false;
        return to.value === "" || hasProtocol(to.value, { acceptRelative: true });
      });
      const prefetched = ref(false);
      const el = import.meta.server ? void 0 : ref(null);
      const elRef = import.meta.server ? void 0 : (
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        (ref2) => {
          el.value = props.custom ? ref2?.$el?.nextElementSibling : ref2?.$el;
        }
      );
      if (import.meta.client) {
        checkPropConflicts(props, "prefetch", "noPrefetch");
        const shouldPrefetch = props.prefetch !== false && props.noPrefetch !== true && props.target !== "_blank" && !isSlowConnection();
        if (shouldPrefetch) {
          const nuxtApp = useNuxtApp();
          let idleId;
          let unobserve = null;
          onMounted(() => {
            const observer = useObserver();
            onNuxtReady(() => {
              idleId = requestIdleCallback(() => {
                if (el?.value?.tagName) {
                  unobserve = observer.observe(
                    el.value,
                    async () => {
                      unobserve?.();
                      unobserve = null;
                      const path = typeof to.value === "string" ? to.value : router.resolve(to.value).fullPath;
                      await Promise.all([
                        nuxtApp.hooks.callHook("link:prefetch", path).catch(() => {
                        }),
                        !isExternal.value && preloadRouteComponents(
                          to.value,
                          router
                        ).catch(() => {
                        })
                      ]);
                      prefetched.value = true;
                    }
                  );
                }
              });
            });
          });
          onBeforeUnmount(() => {
            if (idleId) cancelIdleCallback(idleId);
            unobserve?.();
            unobserve = null;
          });
        }
      }
      return () => {
        if (!isExternal.value) {
          const routerLinkProps = {
            ref: elRef,
            to: to.value,
            activeClass: props.activeClass || options.activeClass,
            exactActiveClass: props.exactActiveClass || options.exactActiveClass,
            replace: props.replace,
            ariaCurrentValue: props.ariaCurrentValue,
            custom: props.custom
          };
          if (!props.custom) {
            if (prefetched.value)
              routerLinkProps.class = props.prefetchedClass || options.prefetchedClass;
            routerLinkProps.rel = props.rel;
          }
          return h(
            resolveComponent("RouterLink"),
            routerLinkProps,
            slots.default
          );
        }
        const href = typeof to.value === "object" ? router.resolve(to.value)?.href ?? null : to.value || null;
        const target = props.target || null;
        checkPropConflicts(props, "noRel", "rel");
        const rel = props.noRel ? null : (
          // converts `""` to `null` to prevent the attribute from being added as empty (`rel=""`)
          firstNonUndefined(
            props.rel,
            options.externalRelAttribute,
            href ? DEFAULT_EXTERNAL_REL_ATTRIBUTE : ""
          ) || null
        );
        const navigate = () => navigateTo(href, { replace: props.replace });
        if (props.custom) {
          if (!slots.default) return null;
          return slots.default({
            href,
            navigate,
            get route() {
              if (!href) return void 0;
              const url = parseURL(href);
              return {
                path: url.pathname,
                fullPath: url.pathname,
                get query() {
                  return parseQuery(url.search);
                },
                hash: url.hash,
                // stub properties for compat with vue-router
                params: {},
                name: void 0,
                matched: [],
                redirectedFrom: void 0,
                meta: {},
                href
              };
            },
            rel,
            target,
            isExternal: isExternal.value,
            isActive: false,
            isExactActive: false
          });
        }
        return h(
          "a",
          {
            ref: el,
            href,
            rel,
            target
          },
          slots.default?.()
        );
      };
    }
  });
}
export default /* @__PURE__ */ defineNuxtLink({ componentName: "NuxtLink" });
function useObserver() {
  if (import.meta.server) return;
  const nuxtApp = useNuxtApp();
  if (nuxtApp._observer) return nuxtApp._observer;
  let observer = null;
  const callbacks = /* @__PURE__ */ new Map();
  const observe = (element, callback) => {
    if (!observer) {
      observer = new IntersectionObserver((entries) => {
        for (const entry of entries) {
          const callback2 = callbacks.get(entry.target);
          const isVisible = entry.isIntersecting || entry.intersectionRatio > 0;
          if (isVisible && callback2) callback2();
        }
      });
    }
    callbacks.set(element, callback);
    observer.observe(element);
    return () => {
      callbacks.delete(element);
      observer.unobserve(element);
      if (callbacks.size === 0) {
        observer.disconnect();
        observer = null;
      }
    };
  };
  const _observer = nuxtApp._observer = {
    observe
  };
  return _observer;
}
function isSlowConnection() {
  if (import.meta.server) return;
  const cn = navigator.connection;
  if (cn && (cn.saveData || /2g/.test(cn.effectiveType))) return true;
  return false;
}
